"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MysqlDialect = void 0;
const mysql_driver_js_1 = require("./mysql-driver.js");
const mysql_query_compiler_js_1 = require("./mysql-query-compiler.js");
const mysql_introspector_js_1 = require("./mysql-introspector.js");
const mysql_adapter_js_1 = require("./mysql-adapter.js");
const object_utils_js_1 = require("../../util/object-utils.js");
/**
 * MySQL dialect that uses the [mysql2](https://github.com/sidorares/node-mysql2#readme) library.
 *
 * The {@link MysqlDialectConfig | configuration} passed to the constructor
 * is given as-is to the mysql2 library's [createPool](https://github.com/sidorares/node-mysql2#using-connection-pools)
 * method.
 */
class MysqlDialect {
    #config;
    constructor(config) {
        this.#config = (0, object_utils_js_1.freeze)({ ...config });
    }
    createDriver() {
        return new mysql_driver_js_1.MysqlDriver(this.#config);
    }
    createQueryCompiler() {
        return new mysql_query_compiler_js_1.MysqlQueryCompiler();
    }
    createAdapter() {
        return new mysql_adapter_js_1.MysqlAdapter();
    }
    createIntrospector(db) {
        return new mysql_introspector_js_1.MysqlIntrospector(db);
    }
}
exports.MysqlDialect = MysqlDialect;
